<?php
declare(strict_types=1);

namespace Inti\Media\Service;

use Imagick;
use ImagickException;
use Inti\Media\Model\ImagePath;
use Inti\Wp\Provider\Service\IdManager;
use Inti\Wp\Provider\Type\Post\Attachment;
use InvalidArgumentException;
use Laminas\Filter\FilterInterface;
use Spatie\ImageOptimizer\OptimizerChainFactory;

use function array_search;
use function dirname;
use function file_exists;
use function file_get_contents;
use function imagecreatefromstring;
use function imagewebp;
use function is_dir;
use function ltrim;
use function mime_content_type;
use function mkdir;
use function preg_match;
use function preg_replace;
use function realpath;

/**
 * Class ImageManager
 *
 * @package Inti\Media\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-media
 */
final class ImageManager
{
    private IdManager       $idManager;
    private array           $ids = [];
    private FilterInterface $pathFilter;
    private string          $root;

    public function __construct(string $root, IdManager $idManager, FilterInterface $pathFilter)
    {
        $this->root = realpath($root);
        $this->idManager = $idManager;
        $this->pathFilter = $pathFilter;

        if (!$this->root) {
            throw new InvalidArgumentException(sprintf(
                '%s: Invalid path `%s` for image root.',
                __METHOD__, $root
            ));
        }
    }

    public function getImageId(string $path): int
    {
        if (preg_match('#^https?://(.+)$#', $path, $matches)) {
            $url = $path;
            $path = $this->pathFilter->filter($matches[1]);

            if (!$this->has($path)) {
                $path = new ImagePath($path);
                $imageData = file_get_contents($url);
                $this->write($path->setExtension('webp')->getRawPath(), $imageData);
            }
        } else {
            $path = new ImagePath($path);
        }

        if (!isset($this->ids[$path->getRawPath()])) {
            $this->ids[$path->getRawPath()] = $this->idManager->offset(count($this->ids) + 1, Attachment::instance());
        }

        return $this->ids[$path->getRawPath()];
    }

    public function getImagePath(int $id): string
    {
        $id = !$this->idManager->isOffset($id)
            ? $this->idManager->offset($id, Attachment::instance())
            : $id;

        return array_search($id, $this->ids) ?: '';
    }

    public function getImageUrl(string $path): string
    {
        return '/media/' . ltrim($path, '/');
    }

    public function getMimeType(string $path): string
    {
        return mime_content_type($this->getFullPath($path));
    }

    public function has(string $path): bool
    {
        $path = new ImagePath($path);
        return file_exists($this->root . '/' . $path->getRawPath());
    }

    public function read(string $path): ?string
    {
        $path = new ImagePath($path);
        $fullPath = $this->getFullPath($path->getFullPath());

        if (file_exists($fullPath)) {
            return file_get_contents($fullPath);
        }

        $rawPath = $this->getFullPath($path->getRawPath());
        $imageData = file_get_contents($rawPath);

        /** @noinspection PhpUnhandledExceptionInspection */
        $resizedData = $this->resize($imageData, $path->getWidth(), $path->getHeight(), $path->isCropped());
        $this->write($path->getFullPath(), $resizedData);

        return $resizedData;
    }

    public function getFullPath(string $path): string
    {
        return preg_replace(
            '/\?.+$/', '',
            $this->root . '/' . ltrim($path, '/')
        );
    }

    public function write(string $path, string $imageData): string
    {
        $fullPath = $this->root . '/' . $path;
        $this->assureDirectoryExists(dirname($fullPath));

        $image = imagecreatefromstring($imageData);
        imagewebp($image, $fullPath, 100);

        $optimizer = OptimizerChainFactory::create();
        $optimizer->optimize($fullPath);

        return $path;
    }

    private function assureDirectoryExists(string $path): void
    {
        if(!is_dir($path)) {
            mkdir($path, 0775, true);
        }
    }

    /**
     * @param string $contents
     * @param int|null $width
     * @param int|null $height
     * @param bool $crop
     * @return string
     * @throws ImagickException
     */
    private function resize(string $contents, int $width = null, int $height = null, bool $crop = false): string
    {
        $imagick = new Imagick();
        $imagick->readImageBlob($contents);

        $imageWidth = $imagick->getImageWidth();
        $imageHeight = $imagick->getImageHeight();
        $imageRatio = $imageWidth / $imageHeight;
        $bestfit = false;

        if (!$width || !$height) {
            if ($width) {
                $newWidth = $width;
                $newHeight = $width / $imageRatio;
            } else {
                $newWidth = $height * $imageRatio;
                $newHeight = $height;
            }
        } else  {
            $bestfit = true;
            $newWidth = $width;
            $newHeight = $height;
        }

        if ($crop) {
            $bestfit = false;
        }

        $imagick->resizeImage(intval($newWidth), intval($newHeight), Imagick::FILTER_UNDEFINED, 1, $bestfit);
        return $imagick->getImageBlob();
    }
}
