<?php
/** @noinspection PhpUnusedPrivateFieldInspection */
declare(strict_types=1);

namespace IntiTest\Media\Provider;

use Inti\Media\ConfigProvider;
use Inti\Media\Service\ImageManager;
use Inti\Test\TestCase\WordPressTestCase;
use WP_Post;

use function file_exists;
use function get_attached_file;
use function get_post;
use function get_post_meta;
use function image_downsize;
use function realpath;
use function wp_get_attachment_url;

/**
 * Class ImageProviderTest
 *
 * @package IntiTest\Media\Provider
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-bartog
 *
 * @runTestsInSeparateProcesses
 */
final class ImageProviderTest extends WordPressTestCase
{
    private array $providers = [
        \Laminas\Filter\ConfigProvider::class,
        \Inti\Filter\ConfigProvider::class,
        \Inti\Provider\ConfigProvider::class,
        \Inti\Wp\Provider\ConfigProvider::class,
        ConfigProvider::class,
    ];

    private array $config = [
        'db_connections' => [
            'default' => [
                'host' => 'localhost',
                'user' => 'root',
                'password' => 'password',
                'dbname' => 'inti_wp_test'
            ],
        ],
        'media' => [
            ImageManager::class => [
                'root' => __DIR__ . '/../media',
            ],
        ],
    ];

    protected ImageManager $imageManager;

    protected function setUp(): void
    {
        parent::setUp();

        $this->runApplication();
        $this->imageManager = $this->_container->get(ImageManager::class);
    }

    public function testCanGetImagePosts()
    {
        $id = $this->imageManager->getImageId('image.webp?hash');

        $post = get_post($id);
        $this->assertInstanceOf(WP_Post::class, $post);
        $this->assertEquals('image.webp?hash', get_post_meta($post->ID, '_wp_attached_file', true));
        $this->assertEquals('image/webp', $post->post_mime_type);
    }

    public function testCanGetAttachmentUrl()
    {
        $id = $this->imageManager->getImageId('image.webp?hash');

        $url = wp_get_attachment_url($id);
        $this->assertEquals('/media/image.webp?hash', $url);
    }

    public function testCanGetAttachmentFile()
    {
        $id = $this->imageManager->getImageId('image.webp?hash');

        $path = get_attached_file($id);
        $this->assertEquals(realpath(__DIR__ . '/../media/image.webp'), $path);
    }

    public function testCanDownsizeImage()
    {
        $id = $this->imageManager->getImageId('image.webp?hash');

        $size = image_downsize($id, 'thumbnail');

        $this->assertIsArray($size);
        $this->assertCount(4, $size);
        $this->assertEquals('/media/thumb/150x150/image.webp?hash', $size[0]);
        $this->assertEquals(150, $size[1]);
        $this->assertEquals(150, $size[2]);
        $this->assertTrue($size[3]);
    }

    public function testCanGetExternalImages()
    {
        $path = realpath(__DIR__ . '/../media') . '/bikez.com/pictures/bmw/1992/k-1.webp';
        if (file_exists($path)) @unlink($path);
        $this->assertFileDoesNotExist($path);

        $id = $this->imageManager->getImageId('https://bikez.com/pictures/bmw/1992/k%201.jpg');
        $this->assertEquals($path, get_attached_file($id));
    }
}
